-- get the addon namespace
local addon, ns = ...
-- get the config values
local cfg = ns.cfg
-- holder for some lib functions
local lib = _G["HEDD_lib"] or CreateFrame("Frame","HEDD_lib")
ns.lib = lib

lib.CD_OnEvent = function(self, event, ...)
	local start, duration = GetSpellCooldown(self.spellID)
    if duration > 0 then
		self.cooldown:Show()
        self.cooldown:SetCooldown(start, duration)
    else
        self.cooldown:Hide()
    end
end

lib.CD_OnEventItem = function(self, event, ...)
	oldtable=hedlib.shallowCopy(self)
	self.start, self.duration, self.enable = GetItemCooldown(self.itemID)
	if not hedlib.shallowCompare(self,oldtable) then return end
	if enable == 0 then
		hedlib.SetDesaturation(self.icon,1)
		print("Disabled")
		return
	end
	hedlib.SetDesaturation(self.icon,nil)
    if self.duration > 0 then
		self.cooldown:Show()
        self.cooldown:SetCooldown(self.start, self.duration)
    else
        self.cooldown:Hide()
    end
end

lib.CDadd = function(spell)
	if not cfg.spells[spell] then return end
	
	local btn = _G["HEDD_CD_"..spell] or CreateFrame("Button", "HEDD_CD_"..spell, Heddframe.CD, "SecureActionButtonTemplate,ActionButtonTemplate")
	btn.spellID=cfg.spells[spell].id
	btn:SetWidth(cfg.isize)
	btn:SetHeight(cfg.isize)
	btn:SetAttribute("*type1", "spell")
	btn:SetAttribute("spell", cfg.spells[spell].name)
	btn:RegisterForClicks("AnyUp")
	btn.icon:SetTexture(cfg.spells[spell].icon)
	btn:ClearAllPoints()
	if not cfg.cd_num then
		btn:SetPoint("TOP",Heddframe.CD,"BOTTOM")
		cfg.cd_num=1
	else
		cfg.cd_num=cfg.cd_num+1
		btn:SetPoint("LEFT",("HEDD_CD_"..cfg.cd_last),"RIGHT",10,0)
	end
	cfg.cd_last=spell
	
	
	local cd = _G["HEDD_CD_"..spell.."Cooldown"] or CreateFrame("Cooldown", "$parentCooldown", btn, "CooldownFrameTemplate")
	cd:SetAllPoints(true)
	cd:SetFrameLevel(4)
	cd:Hide()
	btn.cooldown=cd

	local shine = _G["HEDD_CD_"..spell.."Shine"] or CreateFrame("FRAME", "$parentShine", btn, "SpellBookShineTemplate") --"SpellBookShineTemplate,AutoCastShineTemplate")
	shine:SetFrameStrata("MEDIUM")
	shine:SetFrameLevel(5)
	btn.shine=shine
	AutoCastShine_AutoCastStart(btn.shine)
	
	if HeddDB[btn.spellID] then btn.shine:Hide()
	else btn.shine:Show() end
	btn:Show()
	btn:RegisterEvent("SPELL_UPDATE_COOLDOWN")
	btn:SetScript("OnEvent", lib.CD_OnEvent)
	lib.CD_OnEvent(btn)
	btn:SetScript("PostClick", function(self, button)
	if button~="LeftButton" then 
		lib.CDtoggle(spell)
	end
	end)
end

lib.Itemadd = function(item)
	if not item then return nil end
	local texture=select(10,GetItemInfo(item))
	if not texture then return nil end
	
	local btn = _G["HEDD_CD_"..item] or CreateFrame("Button", "HEDD_CD_"..item, Heddframe.CD, "SecureActionButtonTemplate,ActionButtonTemplate")
	btn.itemID=item
	btn:SetWidth(cfg.isize)
	btn:SetHeight(cfg.isize)
	btn:SetAttribute("*type1", "item")
	btn:SetAttribute("item", "item:"..item)
	btn:RegisterForClicks("AnyUp")
	btn.icon:SetTexture(texture)
	btn:ClearAllPoints()
	if not cfg.cd_num then
		btn:SetPoint("TOP",Heddframe.CD,"BOTTOM")
		cfg.cd_num=1
	else
		cfg.cd_num=cfg.cd_num+1
		btn:SetPoint("LEFT",("HEDD_CD_"..cfg.cd_last),"RIGHT",10,0)
	end
	cfg.cd_last=item
	
	
	local cd = _G["HEDD_CD_"..item.."Cooldown"] or CreateFrame("Cooldown", "$parentCooldown", btn, "CooldownFrameTemplate")
	cd:SetAllPoints(true)
	cd:SetFrameLevel(4)
	cd:Hide()
	btn.cooldown=cd
	btn:Show()
	btn:RegisterEvent("SPELL_UPDATE_COOLDOWN")
	btn:RegisterEvent("PLAYER_REGEN_ENABLED")
	btn:SetScript("OnEvent", lib.CD_OnEventItem)
	lib.CD_OnEventItem(btn)
end

lib.CDturnoff = function(spell)
	if not cfg.spells[spell] then return end
	local btn = _G["HEDD_CD_"..spell]
	if btn then
		btn.shine:Hide()
		HeddDB[btn.spellID]=true
		cfg.Update=true
		btn:SetScript("PostClick",nil)
	end
end

lib.CDtoggleOn = function(spell)
	if not cfg.spells[spell] then return end
	local btn = _G["HEDD_CD_"..spell]
	if btn then
		btn.shine:Show()
		HeddDB[btn.spellID]=false
		cfg.Update=true
		btn:SetScript("PostClick", function(self, button)
			if button~="LeftButton" then 
				lib.CDtoggle(spell)
			end
		end)
	end
end

lib.CDtoggleOff = function(spell)
	if not cfg.spells[spell] then return end
	local btn = _G["HEDD_CD_"..spell]
	if btn then
		btn.shine:Hide()
		HeddDB[btn.spellID]=true
		cfg.Update=true
		btn:SetScript("PostClick", function(self, button)
			if button~="LeftButton" then 
				lib.CDtoggle(spell)
			end
		end)
	end
end

lib.CDtoggle = function(spell)
	if not cfg.spells[spell] then return end
	local btn = _G["HEDD_CD_"..spell]
	if btn then
		if HeddDB[btn.spellID] then
			btn.shine:Show()
			HeddDB[btn.spellID]=nil
			cfg.Update=true
			--print(spell.." show")
		else
			btn.shine:Hide()
			HeddDB[btn.spellID]=true
			cfg.Update=true
			--print(spell.." hide")
		end
	end
end

lib.CDdel = function(spell)
	local btn=_G["HEDD_CD_"..spell]
	if btn then
	--SpellBook_ReleaseAutoCastShine(btn.shine)
	btn:Hide()
	end
end

lib.CDresize = function(spell)
	local btn=_G["HEDD_CD_"..spell]
	if btn then
		btn:SetWidth(cfg.isize)
		btn:SetHeight(cfg.isize)
	end
end